#!/usr/bin/env perl
# Package that reads Zanata configure file.
# Note this modules neither read zanata.ini nor zanata.xml
#
package Zanata::Util::Locale;
use 5.008_008;
use strict;
use File::Basename;
use File::Slurp qw(read_file);
use File::Spec;
use List::MoreUtils qw(uniq);
use Locale::Country;

## Need for handling https
use Data::Dumper;
use Cwd;
my $localeTxt='locale.txt';
my $refFile='';
my %langMap;
my %langCountryMap;

# Split locale and try to fill all the fields.
sub new{
    my $lStr= shift;
    my @lTokens=split(/[_-]/,  $lStr);
    my $self={language=> shift(@lTokens)};

    while(@lTokens){
	my $str=shift(@lTokens);
	if (code2country($str)){
	    $self->{'country'}=$str;
	}else{
	    $self->{'script'}=$str;
	}
    }
    bless $self;
    return $self;
}

sub init{
    my ($workDir, $refFile)=@_;
    unless($workDir){
	$workDir=Cwd::cwd();
    }
    $workDir=Cwd::abs_path($workDir);

    unless($refFile){
	for my $d ("/usr/share/zanata-util",  @INC){
	    if ( -r File::Spec->join($d,$localeTxt)){
		$refFile=File::Spec->join($d,$localeTxt);
		last;
	    }elsif(-r File::Spec->join($d, '..',  'data', $localeTxt)){
		$refFile=File::Spec->join($d, '..',  'data', $localeTxt);
		last;
	    }

	}
    }
    for my $line (read_file($refFile)){
	chomp $line;
	my $locale=new $line;
	my $lang=$locale->{language};
	my $langCountry=join('_', $lang, $locale->{country});
	my $langScript=($locale->{script}) ? join('_', $lang, $locale->{script}): '';
	my $langScriptCountry=($locale->{script}) ? join('_', $langScript, $locale->{country}) : $langCountry;
	unless($langMap{$lang}){
	    $langMap{$lang}=$langScriptCountry;
	}
	unless($langMap{$langCountry}){
	    $langMap{$langCountry}=($langScriptCountry);
	}

	if( $langScript and $langMap{$langScript} eq ''){
	    $langMap{$langScript}=$langScriptCountry;
	    $langMap{$langScriptCountry}=$langScriptCountry;
	}
    }
}

sub is_locale{
    my ($str)=@_;
    return $langMap{$str};
}



sub complete{
    my ($self)= @_;

    ## Already completed if country and script are filled
    if ($self->{script} and $self->{country}){
	return;
    }

    my $pattern=($self->{language});

    if ($self->{script}){
	$pattern=$pattern . "_" . $self->{script};
    }
    if ($self->{country}){
	$pattern=$pattern . "_" . $self->{country};
    }

    my $locale=new $langMap{$pattern};

    for my $i ( 'script', 'country'){
	if ($locale->{$i}){
	    $self->{$i}=$locale->{$i};
	}
    }
}

sub to_string{
    my ($self)= @_;
    if ($self->{script}){
	if ($self->{country}){
	    return join('_', $self->{language}, $self->{script},  $self->{country});
	}
	return join('_', $self->{language}, $self->{script});
    }
    if ($self->{country}){
	return join('_', $self->{language}, $self->{country});
    }
    return $self->{language};
}

sub suggest_locales{
    my ($lStr)=@_;
    my $locale=Zanata::Util::Locale::new $lStr;
    my @localeL=();

    # First try exact match
    push @localeL, $lStr;
    $locale->complete();

    # Then try from longest to shortest
    if ($locale->{script}){
	if ($locale->{country}){
	    push @localeL, $locale->to_string;
	}
	push @localeL, join('_', $locale->{language}, $locale->{script});
    }
    if ($locale->{country}){
	push @localeL, join('_', $locale->{language}, $locale->{country});
    }

    # Make sure lang is actually belong to this locale
    if ($langMap{$locale->{language}} eq $locale->to_string){
	push @localeL, $locale->{language};
    }

    # Remove duplicate
    my @pL=uniq(@localeL);
    #print Dumper(@pL) . "\n";
    return \@pL;
}

sub dir_structure{
    my @dirL=();
    File::Find::find(
	{wanted=>
	    sub{
		if (-d $_ and -x $_){
		    my $rel_path=File::Spec->abs2rel($File::Find::name, $_[0]);
		    push @dirL, $rel_path;
		}
	    }
	    , follow => 1
	}
	, @_
    );
    @dirL=sort @dirL;
    return \@dirL;
}

sub dir_structure_is_equal{
    my ($dirStru1Ref, $dirStru2Ref) = @_;
    my $s1=join('\n', @$dirStru1Ref);
    my $s2=join('\n', @$dirStru2Ref);
    # print "s1=$s1\n";
    # print "s2=$s2\n";
    if ($s1 eq $s2){
	return 1;
    }
    return 0;
}


# dir should be absolute
# mode: 1: file only, 2: dir only, 3: both
# returned contents is related, however.
sub get_valid_dir_contents{
    my ($dir, $mode, $pattern)=@_;
    my @pathL;
    my $D;
    opendir($D, $dir);
    #print "mode=$mode dir=$dir pattern=$pattern\n";
    while(my $f=readdir($D)){
	#print "  f=$f\n";
	### Skip the path start with .
	next if ($f =~ /^\./);

	### Skip CMakeFiles
	next if ($f eq 'CMakeFiles');

	my $p=File::Spec->join($dir,$f);
	next unless ( -r $p);

	### Skip directory in file only mode
	next if ( -d $p and $mode==1);

	### Skip file in directory only mode
	next if ( -f $p and $mode==2);

	### Skip if $f does not match pattern
	if ($pattern){
	    next unless $f =~ $pattern;
	}
	#print " #f=$f pattern=$pattern\n";

	push @pathL, $f;
    }
    closedir($D);
    return \@pathL;
}

=item make_pattern($localeStr, $projType)

Convert locale string to regex pattern

=cut
sub make_pattern{
    my ($localeStr,$projType)=@_;
    $localeStr =~ s/[_-]/[_-]/;
    my $pattern='^' . $localeStr . '(@[A-Za-z0-9]*)?';
    if ($projType eq "gettext"){
	$pattern .= ".po";
    }elsif ($projType =~ ".*properties"){
	$pattern .=".*_${pattern}\.properties";
    }
    $pattern.='$';
    return $pattern ;
}

=item find_locale_paths($localeStr, $dir [, $dirCompare])

Find locale paths.

=over 4

=item localeStr

Locate to be found.

=item dir

Directory to start the search (must be absolute path)

=item dirCompare (Optional)

If specified, contents of found paths must also exist in this directory. (must be absolute path)

=back

=cut
sub find_locale_paths{
    my ($localeStr, $projType, $dir, $dirCompare)=@_;
    my $dirCompareStruRef;

    if ($dirCompare){
	$dirCompareStruRef=dir_structure($dirCompare);
    }

    my $lPattern=make_pattern($localeStr, $projType);

    my @pathL=();
    my @dirQueue=( $dir );
    ## For each directory dirC in dirQueue
    my $validDirContentMode=($dirCompare) ? 2 : 1;
    for my $dirC (@dirQueue){
	#print("dirC=$dirC lPattern=$lPattern dirCompare=$dirCompare\n");
	my $pathLRef=get_valid_dir_contents($dirC, $validDirContentMode, $lPattern);
	my $matchSuccess=0;
	for my $f (@$pathLRef){
	    ### If a path matches $lPattern
	    my $path=File::Spec->join($dirC, $f);
	    if ($f =~ /$lPattern/){
		# print("path=$path lPattern=$lPattern dirCompare=$dirCompare\n");
		#### If the matched is a directory and dirCompare present
		if( -d $path and $dirCompare){
		    ##### If the matched directory and dirCompare has same directory structure
		    if (dir_structure_is_equal($dirCompareStruRef, dir_structure($path))){
			###### then push match to the list
			push @pathL, $f;
			$matchSuccess=1;
		    }
		}else{
		    ##### else implies file mode, push match to the list
		    push @pathL, $f;
		    $matchSuccess=1;
		}
	    }
	    print {*STDERR} "Translation for $localeStr found: $path\n" if $matchSuccess;

	    ### No need to recurse into sub directories
	    last if ($matchSuccess);
	}
	unless($matchSuccess){
	    #### If no match found
	    my $pathLRef=get_valid_dir_contents($dirC, 2);

	    for my $f (@$pathLRef){
		my $p=File::Spec->join($dirC,$f);
		###### then push subdirectories to dirQueue
		push @dirQueue, $p;
	    }
	}
    }
    #print "pathL=" . Dumper(@pathL) ."\n";
    return \@pathL;
}


=item zanata_language_list_parse_locales($url)

Parse the language list from a Zanata server and put the result to a hash.
Language is the key and a reference to an array [ Name, NativeName, Members] is value.

B<Arguments>

=over 4

=item url

Zanata Server url

=back

B<Returns:> A language hash which
Or undef if xml does not contain any locale

=cut
sub zanata_language_list_parse_locales{
    my ($zanataUtil)=@_;
    my %languageH=();

    ## Return undef if  url does not contain language list
    my $hTwig=XML::Twig::XPath->new(pretty_print=>'indented',
	twig_roots=>{q|//table| => 1}
	, twig_handlers=>{ 'tr' => \&zanata_language_list_parse_tr}
    );

    ## Return undef if safe_parseurl failed
    unless($hTwig->safe_parseurl($zanataUtil->{'languageListUrl'})){
	print {*STDERR} "[zanata_language_list_parse_locales] Failed to parse the URL from " . $zanataUtil->{'languageListUrl'} . "\n";
	return undef;
    }

    for my $trElt ($hTwig->findnodes('//tr')){
	my $key;
	my @a;
	for my $tdElt ($trElt->children('td')){
	    my $text=$tdElt->trimmed_text;
	    my $col=$tdElt->att('col');
	    if ($col){
		$a[$col - 1]=$text;
		if ($col>=3){
		    $languageH{$key}=\@a;
		}
	    }else{
		$key=$text;
	    }

	}
    }
    return \%languageH;
}

sub zanata_language_list_parse_tr{
    my ($twig, $tr)=@_;
    my $tdCounter=0;
    my $key='';
    for my $td ($tr->children('td')){
	$td->set_att('col', $tdCounter);
	$tdCounter++;
    }
}

1;
